<?php

namespace Drupal\visualn_libraries_override\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Class OverridesSettingsForm.
 */
class OverridesSettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'visualn_libraries_override.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'visualn_libraries_override_settings_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('visualn_libraries_override.settings');

    $overrides_list = [];
    $overrides = $config->get('overrides');
    if (!empty($overrides)) {
      foreach ($overrides as $item) {
        $overrides_list[] = "{$item['base']}:{$item['override']}";
      }
    }

    $form['overrides_list'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Overrides list'),
      '#description' => $this->t('List of base:override libraries pairs, for example: <em>first_module/library:second_module/another_library</em>'),
      '#default_value' => implode(PHP_EOL, $overrides_list),
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);
  }

  // @todo: validate configuration form

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);

    $overrides = [];
    $overrides_list = $form_state->getValue('overrides_list');
    if (!empty($overrides_list)) {
      $overrides_list = explode(PHP_EOL, $overrides_list);
      foreach ($overrides_list as $item) {
        $item = trim($item);
        // @todo:  check regex [a-zA-Z_]:[a-zA-Z_] (add all other allowed values)
        // @todo: use ":" symbol for delimiter sinces it has special meaning in yml files
        //   and should not be used in libraries names
        if (!empty($item)) {
          list($lib_base, $lib_override) = explode(":", $item);
          $overrides[] = [
            'base' => $lib_base,
            'override' => $lib_override,
            // @todo:
            //'keep_dependencies' => FALSE,
          ];

        }
      }
    }

    // @todo: maybe use string numeric keys for config
    //   also set default value in install config to "{}" then
    $this->config('visualn_libraries_override.settings')
      ->set('overrides', $overrides)
      ->save();

    // clear libraries cache only
    \Drupal::service('library.discovery.collector')->clear();
  }

}
